package org.ow2.choreos.gui.interfaceMappingsVisualizer.components;


import java.awt.Color;
import java.awt.Cursor;
import java.awt.Graphics;
import java.awt.Point;
import java.awt.Rectangle;
import java.awt.event.ActionEvent;
import java.awt.event.MouseEvent;
import java.awt.event.MouseWheelEvent;
import java.util.ArrayList;
import java.util.Random;

import javax.swing.AbstractAction;
import javax.swing.Action;
import javax.swing.GroupLayout;
import javax.swing.JButton;
import javax.swing.JComponent;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JScrollBar;
import javax.swing.JScrollPane;
import javax.swing.JTextArea;
import javax.swing.JTree;
import javax.swing.KeyStroke;
import javax.swing.GroupLayout.Alignment;
import javax.swing.LayoutStyle.ComponentPlacement;
import javax.swing.border.TitledBorder;
import javax.swing.tree.DefaultMutableTreeNode;
import javax.swing.tree.DefaultTreeModel;
import javax.swing.tree.TreePath;
import javax.swing.tree.TreeSelectionModel;

import org.ow2.choreos.abstractionDrivenServiceOrganization.mappings.functionalRepresentation.service.interfaceMappingsToJTrees.TreeUtils;
import org.ow2.choreos.gui.interfaceMappingsVisualizer.listeners.InterfaceMappingsVisualizerListener;
import org.ow2.choreos.gui.interfaceMappingsVisualizer.listeners.MappingsPanelMouseListener;
import org.ow2.choreos.gui.interfaceMappingsVisualizer.listeners.ScrollPaneComponentAdapter;
import org.ow2.choreos.gui.interfaceMappingsVisualizer.listeners.TreeComponentAdapter;
import org.ow2.choreos.gui.interfaceMappingsVisualizer.listeners.TreeExpandOrWillExpandListener;
import org.ow2.choreos.gui.visualComponentConnection.ConnectLine;
import org.ow2.choreos.gui.visualComponentConnection.VisualComponentConnection;
import org.ow2.choreos.gui.visualComponentConnection.VisualComponentConnectionLineProperties;
import org.ow2.choreos.gui.visualComponentConnection.VisualComponentConnectionManagementHistory;
import org.ow2.choreos.gui.visualComponentConnection.VisualComponentConnectionUtils;


/**
 * This class represents a VisualComponentConnection Container that extends {@link JPanel}.
 * <p>This panel is used for representing mappings(connections) between various
 * nodes of two {@link JTree}s. Each mapping is an instance of {@link VisualComponentConnection}
 * class stored in an {@link ArrayList}.
 * <p>This class has two {@code JScrollPanes} with viewports one {@code Jtree} each.
 * By default all listeners needed for handling Mouse Events, resizing, scrolling are added.
 * <p>Also adding and deleting(removing) a {@code VisualComponentConnection} component is already handled.
 * On a side note this class also provides all the needed methods for adding, removing
 * and managing {@code VisualComponentConnection} properties manually.
 * <ul>Extra: <li>For creating random {@code VisualComponentConnection}s see {@see #CreateRandomMappings}
 * <li>Right clicking on a node of one of the {@code JTree}s will trigger a popup menu
 * <li>MouseWheelMoved events outside or between of the two ScrollPanes will scroll both
 * of the ScrollPanes for the same values
 * <li>Clicking with the middle mouse button also enables scrolling for both ScrollPanes</ul>
 */
public class MappingsPanel extends javax.swing.JPanel {
    
    /**
	 * 
	 */
	private static final long serialVersionUID = -5420708414345934664L;
	
    /**An integer indicating the {@code source JTree} ID*/
    public static final int SOURCE_TREE = 0;
    /**An integer indicating the {@code destination JTree} ID*/
    public static final int DESTINATION_TREE = 1;
    
	/**An {@link ArrayList} of existing {@link VisualComponentConnection} components contained in this container*/
    private ArrayList<VisualComponentConnection> existingMappings = new ArrayList<VisualComponentConnection>();
    /**An {@link ArrayList} of {@link VisualComponentConnection} components ready for deletion in this container*/
    private ArrayList<VisualComponentConnection> deletedMappings = new ArrayList<VisualComponentConnection>();
    /**An {@link ArrayList} of {@link VisualComponentConnection} components ready for addition in this container*/
    private ArrayList<VisualComponentConnection> addedMappings = new ArrayList<VisualComponentConnection>();
    
    /**Indicates the current position of the <code>destTree</code> JTree component*/
    private Point destTreeLocation;
    /**Indicates the current position of the <code>sourceTree</code> JTree component*/
    private Point sourceTreeLocation;
    
    /**Indicates the current line type of all the <code>VisualComponentConnection</code>
     * objects contained in this Container
     */
    private int lineType = VisualComponentConnectionLineProperties.CONNECT_LINE_TYPE_RECTANGULAR;
    
    /**Indicates the current line color of existing
     * {@link VisualComponentConnection} components contained in this Container*/
    private Color lineColor = Color.black;
    /**Indicates the current line color of {@link VisualComponentConnection} components ready for deletion
     * in this Container*/
    private Color deletedLineColor = Color.red;
    /**Indicates the current line color of {@link VisualComponentConnection} components ready for addition
     * in this Container*/
    private Color addedLineColor = Color.green;
    
    /**The top left <code>(x,y)</code> coordinates of the <code>JScrollPane</code>
     * containing the <code><strong>sourceTree</strong> Jtree</code>
     */
    public Point sourceScrollPaneLocation;
    /**The top left <code>(x,y)</code> coordinates of the <code>JScrollPane</code>
     * containing the <code><strong>destTree</strong> Jtree</code>
     */
    public Point destScrollPaneLocation;
    
    /**An instance of {@link MappingsManagementPopupMenu} used for this container*/
    private MappingsManagementPopupMenu popupMenu;
    
    /**An instance of {@link VisualComponentConnectionManagementHistory} used for any changes that happened
     * (deletions or additions) on {@link VisualComponentConnection} components  of this 
     * {@link MappingsPanel container}*/
    private VisualComponentConnectionManagementHistory history;

    /*Misc*/
    private int oldMouseY;
    /*Indicates if button2(middle mouse button) has toggled scrolling on both
     scrollPanes* or not. Check formMouserReleased and formMouseMovedc for info*/
    private boolean button2ClickedFlag = false;

    /** Creates new form MappingsPanel 
     * @param callbackOperations 
     * @wbp.parser.constructor*/
    public MappingsPanel(InterfaceMappingsVisualizerListener callbackOperations) {
        initComponents(callbackOperations);
        DefaultTreeModel treeModel = buildDefaultTreeModel();
        this.setSourceTreeModel(treeModel);
        this.setDestTreeModel(treeModel);
    }

    /**
     * Creates new form MappingsPanel with <code>sourceTree</code> as the 
     * source <code>JTree</code>, at the left side of the panel and
     * <code>destTree</code> as the destination <code>JTree</code>,
     * at the right side of the panel
     * 
     * @param sourceTree An instance of <code>JTree</code> class for the source JTtree
     * @param destTree An instance of <code>JTree</code> class for the destination JTree
     */
    public MappingsPanel(JTree sourceTree, JTree destTree, InterfaceMappingsVisualizerListener callbackOperations) {
        initComponents(callbackOperations);
        
        if (sourceTree != null) {
            this.setSourceTreeModel((DefaultTreeModel) sourceTree.getModel());
        }
        if (destTree != null) {
            this.setDestTreeModel((DefaultTreeModel) destTree.getModel());
        }
    }

    /**
     * Creates new form MappingsPanel with the specified {@link VisualComponentConnection}s
     *  <strong>Warning:</strong>Not fully implemented!
     * @param mappings the VisualComponentConnections to be contained in the Container
     */
    public MappingsPanel(ArrayList<VisualComponentConnection> mappings, InterfaceMappingsVisualizerListener callbackOperations) {
        this.existingMappings = mappings;
        initComponents(callbackOperations);
    }

    /** This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the Form Editor.
     */
    /**
     * 
     */
    private void initComponents(final InterfaceMappingsVisualizerListener callbackOperations) {

        popupMenu = new MappingsManagementPopupMenu(this, callbackOperations);
        history = new VisualComponentConnectionManagementHistory(this);
        sourceTreeScrollPane = new javax.swing.JScrollPane();
        sourceTree = new javax.swing.JTree();
        completeButton = new javax.swing.JButton();
        sourceViewLabel = new javax.swing.JLabel();
        destTreeScrollPane = new javax.swing.JScrollPane();
        destTree = new javax.swing.JTree();
        targetViewLabel = new javax.swing.JLabel();
        expandSourceTreeButton = new javax.swing.JButton();
        collapseSourceTreeButton = new javax.swing.JButton();
        collapseDestTreeButton = new javax.swing.JButton();
        expandDestTreeButton = new javax.swing.JButton();
        undoButton = new javax.swing.JButton();
        redoButton = new javax.swing.JButton();
        mappingsColorChooser = new org.ow2.choreos.gui.interfaceMappingsVisualizer.components.MappingsLineColorChooser();

        setPreferredSize(new java.awt.Dimension(800, 600));
        addMouseWheelListener(new java.awt.event.MouseWheelListener() {
            public void mouseWheelMoved(java.awt.event.MouseWheelEvent evt) {
                formMouseWheelMoved(evt);
            }
        });
        addMouseListener(new java.awt.event.MouseAdapter() {
            public void mouseReleased(java.awt.event.MouseEvent evt) {
                formMouseReleased(evt);
            }
        });
        addMouseMotionListener(new java.awt.event.MouseMotionAdapter() {
            public void mouseMoved(java.awt.event.MouseEvent evt) {
                formMouseMoved(evt);
            }
        });

        sourceTreeScrollPane.addComponentListener(new ScrollPaneComponentAdapter(this));
        
        sourceTreeScrollPane.setPreferredSize(new java.awt.Dimension(200, 322));

        sourceTree.getSelectionModel().setSelectionMode(TreeSelectionModel.SINGLE_TREE_SELECTION);
        
        sourceTree.addTreeExpansionListener(new TreeExpandOrWillExpandListener(this));

        sourceTree.addTreeWillExpandListener(new TreeExpandOrWillExpandListener(this));
        
        sourceTree.addMouseListener(new MappingsPanelMouseListener(callbackOperations,popupMenu,this));
        
        sourceTree.addComponentListener(new TreeComponentAdapter(this));
        
//        TreeUtils.ExpandJTree(this.sourceTree);
        
        sourceTreeScrollPane.setViewportView(sourceTree);

        completeButton.setIcon(new javax.swing.ImageIcon("Icons/Icon.png")); // NOI18N
        completeButton.setText("Complete Verification");
        completeButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                complete(callbackOperations);
            }
        });

        sourceViewLabel.setText("Source will appear here:");

        destTreeScrollPane.addComponentListener(new ScrollPaneComponentAdapter(this));
        
        destTreeScrollPane.setPreferredSize(new java.awt.Dimension(200, 322));

        destTree.getSelectionModel().setSelectionMode(TreeSelectionModel.SINGLE_TREE_SELECTION);
        
        destTree.addTreeExpansionListener(new TreeExpandOrWillExpandListener(this));
        
        destTree.addTreeWillExpandListener(new TreeExpandOrWillExpandListener(this));

        destTree.addMouseListener(new MappingsPanelMouseListener(callbackOperations,popupMenu,this));
        
        destTree.addComponentListener(new TreeComponentAdapter(this));
        
//        TreeUtils.ExpandJTree(this.destTree);
        
        destTreeScrollPane.setViewportView(destTree);

        targetViewLabel.setText("Target will appear here:");

        expandSourceTreeButton.setIcon(new javax.swing.ImageIcon("Icons/expandIcon.png")); // NOI18N
        expandSourceTreeButton.setToolTipText("Expand Left Tree");
        expandSourceTreeButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                expandSourceTreeButtonActionPerformed(evt);
            }
        });

        collapseSourceTreeButton.setIcon(new javax.swing.ImageIcon("Icons/collapseIcon.png")); // NOI18N
        collapseSourceTreeButton.setToolTipText("Collapse Left Tree");
        collapseSourceTreeButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                collapseSourceTreeButtonActionPerformed(evt);
            }
        });

        collapseDestTreeButton.setIcon(new javax.swing.ImageIcon("Icons/collapseIcon.png")); // NOI18N
        collapseDestTreeButton.setToolTipText("Collapse Right Tree");
        collapseDestTreeButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                collapseDestTreeButtonActionPerformed(evt);
            }
        });

        expandDestTreeButton.setIcon(new javax.swing.ImageIcon("Icons/expandIcon.png")); // NOI18N
        expandDestTreeButton.setToolTipText("Expand Right Tree");
        expandDestTreeButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                expandDestTreeButtonActionPerformed(evt);
            }
        });

        undoButton.setIcon(new javax.swing.ImageIcon("Icons/undoIcon.png")); // NOI18N
        undoButton.setEnabled(false);
        undoButton.setRolloverIcon(new javax.swing.ImageIcon("Icons/undoOverIcon.png")); // NOI18N
        undoButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                undoButtonActionPerformed(evt);
            }
        });

        redoButton.setIcon(new javax.swing.ImageIcon("Icons/redoIcon.png")); // NOI18N
        redoButton.setEnabled(false);
        redoButton.setRolloverIcon(new javax.swing.ImageIcon("Icons/redoOverIcon.png")); // NOI18N
        redoButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                redoButtonActionPerformed(evt);
            }
        });

        mappingsColorChooser.setBorder(new TitledBorder(null, "Colors", TitledBorder.LEADING, TitledBorder.TOP, null, null));
        mappingsColorChooser.setAddedColor(java.awt.Color.green);
        mappingsColorChooser.setDeletedColor(java.awt.Color.red);
        mappingsColorChooser.setExistingColor(java.awt.Color.black);

        javax.swing.GroupLayout layout = new javax.swing.GroupLayout(this);
        layout.setHorizontalGroup(
        	layout.createParallelGroup(Alignment.TRAILING)
        		.addGroup(layout.createSequentialGroup()
        			.addContainerGap()
        			.addGroup(layout.createParallelGroup(Alignment.LEADING)
        				.addComponent(expandSourceTreeButton, GroupLayout.PREFERRED_SIZE, 26, GroupLayout.PREFERRED_SIZE)
        				.addComponent(collapseSourceTreeButton, GroupLayout.PREFERRED_SIZE, 26, GroupLayout.PREFERRED_SIZE))
        			.addPreferredGap(ComponentPlacement.RELATED)
        			.addGroup(layout.createParallelGroup(Alignment.LEADING)
        				.addGroup(layout.createSequentialGroup()
        					.addComponent(mappingsColorChooser, GroupLayout.PREFERRED_SIZE, GroupLayout.DEFAULT_SIZE, GroupLayout.PREFERRED_SIZE)
        					.addPreferredGap(ComponentPlacement.RELATED)
        					.addComponent(completeButton, GroupLayout.DEFAULT_SIZE, 535, Short.MAX_VALUE))
        				.addGroup(layout.createSequentialGroup()
        					.addGroup(layout.createParallelGroup(Alignment.LEADING)
        						.addGroup(layout.createSequentialGroup()
        							.addComponent(sourceTreeScrollPane, GroupLayout.DEFAULT_SIZE, 321, Short.MAX_VALUE)
        							.addPreferredGap(ComponentPlacement.RELATED)
        							.addComponent(undoButton, GroupLayout.PREFERRED_SIZE, 26, GroupLayout.PREFERRED_SIZE)
        							.addPreferredGap(ComponentPlacement.RELATED)
        							.addComponent(redoButton, GroupLayout.PREFERRED_SIZE, 26, GroupLayout.PREFERRED_SIZE))
        						.addGroup(layout.createSequentialGroup()
        							.addComponent(sourceViewLabel, GroupLayout.PREFERRED_SIZE, 352, GroupLayout.PREFERRED_SIZE)
        							.addGap(0, 33, Short.MAX_VALUE)))
        					.addPreferredGap(ComponentPlacement.RELATED)
        					.addGroup(layout.createParallelGroup(Alignment.TRAILING)
        						.addGroup(layout.createSequentialGroup()
        							.addComponent(targetViewLabel, GroupLayout.DEFAULT_SIZE, 160, Short.MAX_VALUE)
        							.addGap(165))
        						.addComponent(destTreeScrollPane, GroupLayout.DEFAULT_SIZE, 325, Short.MAX_VALUE))))
        			.addPreferredGap(ComponentPlacement.RELATED)
        			.addGroup(layout.createParallelGroup(Alignment.TRAILING)
        				.addComponent(expandDestTreeButton, GroupLayout.PREFERRED_SIZE, 26, GroupLayout.PREFERRED_SIZE)
        				.addComponent(collapseDestTreeButton, GroupLayout.PREFERRED_SIZE, 26, GroupLayout.PREFERRED_SIZE))
        			.addContainerGap())
        );
        layout.setVerticalGroup(
        	layout.createParallelGroup(Alignment.LEADING)
        		.addGroup(layout.createSequentialGroup()
        			.addGap(19)
        			.addGroup(layout.createParallelGroup(Alignment.BASELINE)
        				.addComponent(sourceViewLabel)
        				.addComponent(targetViewLabel))
        			.addPreferredGap(ComponentPlacement.RELATED)
        			.addGroup(layout.createParallelGroup(Alignment.LEADING)
        				.addComponent(destTreeScrollPane, GroupLayout.DEFAULT_SIZE, 473, Short.MAX_VALUE)
        				.addGroup(layout.createSequentialGroup()
        					.addGroup(layout.createParallelGroup(Alignment.LEADING)
        						.addGroup(layout.createSequentialGroup()
        							.addComponent(expandSourceTreeButton)
        							.addPreferredGap(ComponentPlacement.RELATED)
        							.addComponent(collapseSourceTreeButton))
        						.addGroup(layout.createSequentialGroup()
        							.addComponent(expandDestTreeButton)
        							.addPreferredGap(ComponentPlacement.RELATED)
        							.addComponent(collapseDestTreeButton)))
        					.addPreferredGap(ComponentPlacement.RELATED, 440, Short.MAX_VALUE)
        					.addGroup(layout.createParallelGroup(Alignment.LEADING)
        						.addComponent(undoButton)
        						.addComponent(redoButton)))
        				.addComponent(sourceTreeScrollPane, GroupLayout.DEFAULT_SIZE, 473, Short.MAX_VALUE))
        			.addPreferredGap(ComponentPlacement.RELATED)
        			.addGroup(layout.createParallelGroup(Alignment.TRAILING)
        				.addComponent(mappingsColorChooser, GroupLayout.PREFERRED_SIZE, GroupLayout.DEFAULT_SIZE, GroupLayout.PREFERRED_SIZE)
        				.addComponent(completeButton))
        			.addContainerGap())
        );
        this.setLayout(layout);
        this.addComponentListener(new TreeComponentAdapter(this));
        initLocations();
        initUndoRedoActions();
        initLocations();
        initUndoRedoActions();
        initLineColors();
    }// </editor-fold>//GEN-END:initComponents

    /**
     * Initializes the <code>destTreeScrollPaneLocation, sourceTreeScrollPaneLocation,
     * destTreeLocation and sourceTreeLocation</code> to their appropriate values
     */
    private void initLocations() {
        this.destScrollPaneLocation = this.destTreeScrollPane.getLocation();
        this.destTreeLocation = this.destTree.getLocation();
        this.sourceScrollPaneLocation = this.sourceTreeScrollPane.getLocation();
        this.sourceTreeLocation = this.sourceTree.getLocation();
    }

    /**
     * Initializes the line colors of existing,added and delete {@link VisualComponentConnection}
     * components
     */
    private void initLineColors(){
        lineColor = mappingsColorChooser.getExistingColor();
        addedLineColor = mappingsColorChooser.getAddedColor();
        deletedLineColor = mappingsColorChooser.getDeletedColor();
    }
    /**
     * Binds {@code CTRL+Z and CTRL+Y} {@link KeyStroke}s (for the undo and redo 
     * respectively) to the appropriate {@link Action}s 
     */
    private void initUndoRedoActions() {
        Action undoAction = new AbstractAction() {

            /**
			 * 
			 */
			private static final long serialVersionUID = -8171522399750214948L;

			public void actionPerformed(ActionEvent e) {
                undoButtonActionPerformed(null);
            }
        };

        Action redoAction = new AbstractAction() {

            /**
			 * 
			 */
			private static final long serialVersionUID = -2762415804312661502L;

			public void actionPerformed(ActionEvent e) {
                redoButtonActionPerformed(null);
            }
        };
        KeyStroke undoKeyStroke = KeyStroke.getKeyStroke(java.awt.event.KeyEvent.VK_Z, java.awt.event.KeyEvent.CTRL_DOWN_MASK);
        KeyStroke redoKeyStroke = KeyStroke.getKeyStroke(java.awt.event.KeyEvent.VK_Y, java.awt.event.KeyEvent.CTRL_DOWN_MASK);

        this.getInputMap(JComponent.WHEN_ANCESTOR_OF_FOCUSED_COMPONENT).put(undoKeyStroke,"Undo");
        this.getActionMap().put("Undo", undoAction);

        this.getInputMap(JComponent.WHEN_ANCESTOR_OF_FOCUSED_COMPONENT).put(redoKeyStroke,"Redo");
        this.getActionMap().put("Redo", redoAction);
    }

    public void complete(InterfaceMappingsVisualizerListener callbackOperations) {
    	if(callbackOperations != null)
    		callbackOperations.complete();
    	else{
			this.deletedMappings.clear();
			this.existingMappings.addAll(this.addedMappings);
			for (int i = 0; i < this.existingMappings.size(); i++) {
				this.existingMappings.get(i).setLineColor(this.lineColor);
			}
			this.addedMappings.clear();
			this.history.ClearHistory();
			this.repaint();
		}
}

    private void expandSourceTreeButtonActionPerformed(java.awt.event.ActionEvent evt) {
        TreeUtils.ExpandJTree(this.sourceTree);
    }

    private void collapseSourceTreeButtonActionPerformed(java.awt.event.ActionEvent evt) {
        TreeUtils.CollapseJTree(this.sourceTree);
    }

    private void expandDestTreeButtonActionPerformed(java.awt.event.ActionEvent evt) {
        TreeUtils.ExpandJTree(this.destTree);
    }

    private void collapseDestTreeButtonActionPerformed(java.awt.event.ActionEvent evt) {
        TreeUtils.CollapseJTree(this.destTree);
    }

    private void formMouseWheelMoved(java.awt.event.MouseWheelEvent evt) {
        JScrollBar sourceScrollBar = sourceTreeScrollPane.getVerticalScrollBar();
        JScrollBar destScrollBar = destTreeScrollPane.getVerticalScrollBar();
        int direction = evt.getWheelRotation();
        int sourceIncrement, destIncrement;
        if (evt.getScrollType() == MouseWheelEvent.WHEEL_UNIT_SCROLL) {
            destIncrement = destScrollBar.getUnitIncrement(1);
            sourceIncrement = sourceScrollBar.getUnitIncrement(1);
        } else {
            destIncrement = destScrollBar.getBlockIncrement(1);
            sourceIncrement = sourceScrollBar.getBlockIncrement(1);
        }
        destScrollBar.setValue(destScrollBar.getValue() + destIncrement * direction);
        sourceScrollBar.setValue(sourceScrollBar.getValue() + sourceIncrement * direction);
    }

    private void undoButtonActionPerformed(java.awt.event.ActionEvent evt) {
        if (history.getUndoStack().isEmpty()) {
            return;
        }
        history.ExecuteUndo();
    }

    private void redoButtonActionPerformed(java.awt.event.ActionEvent evt) {
        if (history.getRedoStack().isEmpty()) {
            return;
        }
        history.ExecuteRedo();
    }

    private void formMouseMoved(java.awt.event.MouseEvent evt) {
        if (button2ClickedFlag) {
            if (oldMouseY == 0) {
                oldMouseY = evt.getY();
                return;
            }
            int diff = evt.getY() - oldMouseY;
            oldMouseY = evt.getY();
            sourceTreeScrollPane.getVerticalScrollBar().setValue(sourceTreeScrollPane.getVerticalScrollBar().getValue()+diff);
            destTreeScrollPane.getVerticalScrollBar().setValue(destTreeScrollPane.getVerticalScrollBar().getValue()+diff);
        }
    }

    private void formMouseReleased(java.awt.event.MouseEvent evt) {
        oldMouseY = 0;
        if (evt.getModifiers() == (MouseEvent.ALT_MASK | MouseEvent.BUTTON2_MASK)){
            button2ClickedFlag = !button2ClickedFlag;
            if(button2ClickedFlag) this.setCursor(new Cursor(Cursor.MOVE_CURSOR));
            else setCursor(null);
        }
        else {
            button2ClickedFlag = false;
            setCursor(null);
        }
    }
    
    private DefaultTreeModel buildDefaultTreeModel(){
    	javax.swing.tree.DefaultMutableTreeNode treeNode1 = new javax.swing.tree.DefaultMutableTreeNode("JTree");
        javax.swing.tree.DefaultMutableTreeNode treeNode2 = new javax.swing.tree.DefaultMutableTreeNode("colors");
        javax.swing.tree.DefaultMutableTreeNode treeNode3 = new javax.swing.tree.DefaultMutableTreeNode("blue");
        treeNode2.add(treeNode3);
        treeNode3 = new javax.swing.tree.DefaultMutableTreeNode("violet");
        treeNode2.add(treeNode3);
        treeNode3 = new javax.swing.tree.DefaultMutableTreeNode("red");
        treeNode2.add(treeNode3);
        treeNode3 = new javax.swing.tree.DefaultMutableTreeNode("yellow");
        treeNode2.add(treeNode3);
        treeNode3 = new javax.swing.tree.DefaultMutableTreeNode("a");
        treeNode2.add(treeNode3);
        treeNode3 = new javax.swing.tree.DefaultMutableTreeNode("b");
        treeNode2.add(treeNode3);
        treeNode3 = new javax.swing.tree.DefaultMutableTreeNode("c");
        treeNode2.add(treeNode3);
        treeNode3 = new javax.swing.tree.DefaultMutableTreeNode("d");
        treeNode2.add(treeNode3);
        treeNode3 = new javax.swing.tree.DefaultMutableTreeNode("e");
        treeNode2.add(treeNode3);
        treeNode3 = new javax.swing.tree.DefaultMutableTreeNode("f");
        treeNode2.add(treeNode3);
        treeNode3 = new javax.swing.tree.DefaultMutableTreeNode("g");
        treeNode2.add(treeNode3);
        treeNode3 = new javax.swing.tree.DefaultMutableTreeNode("h");
        treeNode2.add(treeNode3);
        treeNode3 = new javax.swing.tree.DefaultMutableTreeNode("i");
        treeNode2.add(treeNode3);
        treeNode1.add(treeNode2);
        treeNode2 = new javax.swing.tree.DefaultMutableTreeNode("sports");
        treeNode3 = new javax.swing.tree.DefaultMutableTreeNode("basketball");
        treeNode2.add(treeNode3);
        treeNode3 = new javax.swing.tree.DefaultMutableTreeNode("soccer");
        treeNode2.add(treeNode3);
        treeNode3 = new javax.swing.tree.DefaultMutableTreeNode("football");
        treeNode2.add(treeNode3);
        treeNode3 = new javax.swing.tree.DefaultMutableTreeNode("hockey");
        treeNode2.add(treeNode3);
        treeNode1.add(treeNode2);
        treeNode2 = new javax.swing.tree.DefaultMutableTreeNode("food");
        treeNode3 = new javax.swing.tree.DefaultMutableTreeNode("hot dogs");
        treeNode2.add(treeNode3);
        treeNode3 = new javax.swing.tree.DefaultMutableTreeNode("pizza");
        treeNode2.add(treeNode3);
        treeNode3 = new javax.swing.tree.DefaultMutableTreeNode("ravioli");
        treeNode2.add(treeNode3);
        treeNode3 = new javax.swing.tree.DefaultMutableTreeNode("bananas");
        treeNode2.add(treeNode3);
        treeNode1.add(treeNode2);
        treeNode2 = new javax.swing.tree.DefaultMutableTreeNode("drinks");
        treeNode3 = new javax.swing.tree.DefaultMutableTreeNode("spirits");
        javax.swing.tree.DefaultMutableTreeNode treeNode4 = new javax.swing.tree.DefaultMutableTreeNode("vodka");
        treeNode3.add(treeNode4);
        treeNode4 = new javax.swing.tree.DefaultMutableTreeNode("rum");
        treeNode3.add(treeNode4);
        treeNode4 = new javax.swing.tree.DefaultMutableTreeNode("wine");
        treeNode3.add(treeNode4);
        treeNode4 = new javax.swing.tree.DefaultMutableTreeNode("tsipouro");
        treeNode3.add(treeNode4);
        treeNode2.add(treeNode3);
        treeNode3 = new javax.swing.tree.DefaultMutableTreeNode("beverages");
        treeNode4 = new javax.swing.tree.DefaultMutableTreeNode("juice");
        treeNode3.add(treeNode4);
        treeNode4 = new javax.swing.tree.DefaultMutableTreeNode("tea");
        treeNode3.add(treeNode4);
        treeNode4 = new javax.swing.tree.DefaultMutableTreeNode("warm chocolate");
        treeNode3.add(treeNode4);
        treeNode4 = new javax.swing.tree.DefaultMutableTreeNode("sodas");
        javax.swing.tree.DefaultMutableTreeNode treeNode5 = new javax.swing.tree.DefaultMutableTreeNode("coca cola");
        treeNode4.add(treeNode5);
        treeNode5 = new javax.swing.tree.DefaultMutableTreeNode("pepsi cola");
        treeNode4.add(treeNode5);
        treeNode5 = new javax.swing.tree.DefaultMutableTreeNode("fanta");
        treeNode4.add(treeNode5);
        treeNode5 = new javax.swing.tree.DefaultMutableTreeNode("sprite");
        treeNode4.add(treeNode5);
        treeNode5 = new javax.swing.tree.DefaultMutableTreeNode("7up");
        treeNode4.add(treeNode5);
        treeNode3.add(treeNode4);
        treeNode4 = new javax.swing.tree.DefaultMutableTreeNode("coffees");
        treeNode5 = new javax.swing.tree.DefaultMutableTreeNode("frappe");
        treeNode4.add(treeNode5);
        treeNode5 = new javax.swing.tree.DefaultMutableTreeNode("capuccino");
        treeNode4.add(treeNode5);
        treeNode5 = new javax.swing.tree.DefaultMutableTreeNode("fredo");
        treeNode4.add(treeNode5);
        treeNode5 = new javax.swing.tree.DefaultMutableTreeNode("traditional");
        treeNode4.add(treeNode5);
        treeNode3.add(treeNode4);
        treeNode2.add(treeNode3);
        treeNode1.add(treeNode2);
        treeNode2 = new javax.swing.tree.DefaultMutableTreeNode("A");
        treeNode3 = new javax.swing.tree.DefaultMutableTreeNode("Aa");
        treeNode2.add(treeNode3);
        treeNode3 = new javax.swing.tree.DefaultMutableTreeNode("Ab");
        treeNode2.add(treeNode3);
        treeNode3 = new javax.swing.tree.DefaultMutableTreeNode("Ac");
        treeNode4 = new javax.swing.tree.DefaultMutableTreeNode("Aca");
        treeNode3.add(treeNode4);
        treeNode4 = new javax.swing.tree.DefaultMutableTreeNode("Acb");
        treeNode3.add(treeNode4);
        treeNode4 = new javax.swing.tree.DefaultMutableTreeNode("Acc");
        treeNode3.add(treeNode4);
        treeNode4 = new javax.swing.tree.DefaultMutableTreeNode("Acd");
        treeNode5 = new javax.swing.tree.DefaultMutableTreeNode("Acda");
        treeNode4.add(treeNode5);
        treeNode5 = new javax.swing.tree.DefaultMutableTreeNode("Acdb");
        javax.swing.tree.DefaultMutableTreeNode treeNode6 = new javax.swing.tree.DefaultMutableTreeNode("Acdba");
        treeNode5.add(treeNode6);
        treeNode6 = new javax.swing.tree.DefaultMutableTreeNode("Acdbb");
        treeNode5.add(treeNode6);
        treeNode4.add(treeNode5);
        treeNode3.add(treeNode4);
        treeNode4 = new javax.swing.tree.DefaultMutableTreeNode("Acf");
        treeNode5 = new javax.swing.tree.DefaultMutableTreeNode("Acfa");
        treeNode4.add(treeNode5);
        treeNode5 = new javax.swing.tree.DefaultMutableTreeNode("Acfb");
        treeNode4.add(treeNode5);
        treeNode5 = new javax.swing.tree.DefaultMutableTreeNode("Acfa");
        treeNode4.add(treeNode5);
        treeNode3.add(treeNode4);
        treeNode2.add(treeNode3);
        treeNode1.add(treeNode2);
        
        return new DefaultTreeModel(treeNode1);
    }

    public void createRandomMappings() {
        TreeUtils.ExpandJTree(this.sourceTree);
        TreeUtils.ExpandJTree(this.destTree);

        Random random = new Random();
        int connectionsNum = random.nextInt(5) + 5;

        for (int i = 0; i < connectionsNum; i++) {
            TreePath sourcePath = this.sourceTree.getPathForRow(
                    random.nextInt(this.sourceTree.getRowCount()));
            TreePath destPath = this.destTree.getPathForRow(
                    random.nextInt(this.destTree.getRowCount()));
            this.addMapping(sourcePath, destPath);
        }

        VisualComponentConnectionUtils.updateConnectionsVisibility(this,VisualComponentConnectionUtils.DESTINATION);
        VisualComponentConnectionUtils.updateConnectionsVisibility(this,VisualComponentConnectionUtils.SOURCE);
        this.repaint();
    }

    /**
     * Adds <code>newMapping</code> in the {@link ArrayList} of {@link VisualComponentConnection} objects
     *
     * @param newMapping The VisualComponentConnection component to be added
     */
    public void addMapping(VisualComponentConnection newMapping) {
        this.existingMappings.add(newMapping);
        this.history.clearRedoStack();
    }

    /**
     *
     * Adds a new {@link VisualComponentConnection} with the Rectangle containing the last component
     * of <code>sourcePath</code> as source and the Rectangle containing the
     * last component of <code>destPath</code> as destination.
     * Sets the sourcePath and destPath fields of VisualComponentConnection to the specified
     * <code>sourcePath</code> and <code>destPath</code>
     *
     * @param sourcePath The Source TreePath of the mapping
     * @param destPath The Destination TreePath of the mapping
     */
    public void addMapping(TreePath sourcePath, TreePath destPath) {
        if ((sourcePath != null) && (destPath != null)) {
            ArrayList<VisualComponentConnection> foundConnections = VisualComponentConnectionUtils.findConnectionsWith(this, VisualComponentConnectionUtils.ALL_CONNECTIONS,
                    (DefaultMutableTreeNode) sourcePath.getLastPathComponent(),
                    (DefaultMutableTreeNode) destPath.getLastPathComponent());

            //If connection with sourcePath as source and destPath as dest not found
            if (foundConnections.isEmpty()) {
                this.addMapping(this.createNewMapping(sourcePath, destPath));
            }
        }
    }

    /**
     * Removes a {@link VisualComponentConnection} component from the {@code existing Mappings} {@link ArrayList}
     * of this Container. 
     * <P>
     * Note: If no such {@code VisualComponentConnection} exists in the ArrayList nothing is done.
     * @param mapping An instance of {@code VisualComponentConnection} indicating the VisualComponentConnection
     * to remove. 
     */
    public void removeMapping(VisualComponentConnection mapping) {
        this.existingMappings.remove(mapping);
        this.repaint();
    }

    public void addToDeletedMappings(VisualComponentConnection mapping) {
        /*If the specified VisualComponentConnection component is in the existing VisualComponentConnections list
         *then remove it and add it to the VisualComponentConnections for deletion list*/
        if (this.existingMappings.remove(mapping)) {
            mapping.setLineColor(deletedLineColor);
            this.deletedMappings.add(mapping);
            this.history.pushIntoUndo(VisualComponentConnectionManagementHistory.DELETING_EXISTING_CONNECTION, mapping);
            this.history.clearRedoStack();
        }
        /*If the specified VisualComponentConnection component is not in the existing VisualComponentConnections list
         *but it's in the VisualComponentConnections to be added list, then simply remove it*/
        if (this.addedMappings.remove(mapping)) {
            this.history.pushIntoUndo(VisualComponentConnectionManagementHistory.CANCELING_ADDITION_OF_CONNECTION, mapping);
            this.history.clearRedoStack();
        }
        this.repaint();
    }

    public void addToAddedMappings(VisualComponentConnection mapping) {
        /*If VisualComponentConnection connection isn't null and isn't already an existing VisualComponentConnection
         *or a VisualComponentConnection ready for deletion or addition, then add it to connectionsToAdd ArrayList*/
        if ((mapping != null)
                && !VisualComponentConnectionUtils.connectionIsContained(this, VisualComponentConnectionUtils.ALL_CONNECTIONS, mapping))
        {   
            /*Change the color property of the specified VisualComponentConnection component to the apropriate Color value*/
            mapping.setLineColor(addedLineColor);
            /*Add specified VisualComponentConnection component to VisualComponentConnections ready for adding list*/
            this.addedMappings.add(mapping);
            this.history.clearRedoStack();
            this.history.pushIntoUndo(VisualComponentConnectionManagementHistory.ADDING_NEW_CONNECTION, mapping);
            
            VisualComponentConnectionUtils.updateConnectionsVisibility(this,VisualComponentConnectionUtils.SOURCE);
            VisualComponentConnectionUtils.updateConnectionsVisibility(this,VisualComponentConnectionUtils.DESTINATION);
            
        }
    }

    /**
     * Creates and returns a new default {@code VisualComponentConnection} component with {@code source's TreePath}
     * last compomonent as source and with {@code dest's TreePath}
     * last compomonent as destination.
     * @param source An instance of {@code TreePath} indicating the source of the new {@code  VisualComponentConnection}
     * @param dest An instance of {@code TreePath} indicating the destination of the new {@code  VisualComponentConnection}
     * @return A new {@code VisualComponentConnection}
     */
    public VisualComponentConnection createNewMapping(TreePath source, TreePath dest) {
        if (source != null && dest != null) {
            Rectangle rDest = destTree.getPathBounds(dest);
            Rectangle rSource = sourceTree.getPathBounds(source);

            VisualComponentConnection newMapping = new VisualComponentConnection(
                    rSource, rDest,
                    ConnectLine.LINE_ARROW_DEST,
                    this.lineType,
                    this.lineColor,
                    true);
            newMapping.moveConnectionBy(this.sourceScrollPaneLocation, this.destScrollPaneLocation);
            newMapping.moveConnectionBy(this.sourceTreeLocation, this.destTreeLocation);

            newMapping.setPaths(source, dest);


            return newMapping;
        } else {
            return null;
        }
    }

    /*Getters & Setters*/
    /**
     * @param mappingIndex An integer indicating the index of the desired {@link VisualComponentConnection}
     * @return The <code>VisualComponentConnection</code> at <code>mappingIndex</code> of the VisualComponentConnections array List.
     * Returns null if no VisualComponentConnection component exists in this Container
     */
    public VisualComponentConnection getExistingMapping(int mappingIndex) {
        if (!this.existingMappings.isEmpty()) {
            try {
                return this.existingMappings.get(mappingIndex);
            } catch (IndexOutOfBoundsException e) {
                JOptionPane.showMessageDialog(this, "Mapping Index out of Bounds!", "Error!", JOptionPane.ERROR_MESSAGE);
                return null;
            }
        } else {
            return null;
        }
    }

    /**
     * <strong>Note</strong>: Not used. Old version, might need some tweaks..
     * <p>Sets the source and destination Rectangle of the specified {@link VisualComponentConnection} to
     * <code>source</code> and <code>dest</code>
     * 
     * @param pos The location of a {@code VisualComponentConnection} component in the {@link ArrayList} of {@code VisualComponentConnection}s in this Container
     * @param source The new source Rectangle 
     * @param dest The new destination Rectangle
     * @param visible The visibility state of the Mapping
     */
    public void setExistingMapping(int pos, Rectangle source, Rectangle dest, boolean visible) {
        if (!existingMappings.isEmpty()) {
            try {
                this.existingMappings.get(pos).setSourceDest(source, dest, visible);
            } catch (IndexOutOfBoundsException e) {
                JOptionPane.showMessageDialog(this, "Mapping Index out of Bounds!", "Error!", JOptionPane.ERROR_MESSAGE);
            }
            this.repaint();
        }
    }

    /**
     * Returns the {@link VisualComponentConnection} components of this Container.
     * @return An {@link ArrayList} of {@code VisualComponentConnection} Objects
     */
    public ArrayList<VisualComponentConnection> getExistingMappings() {
        return this.existingMappings;
    }

    /**
     * Sets the {@link VisualComponentConnection} {@link ArrayList} of this Container to <code>mappings</code>
     * 
     * @param mappings An {@code ArrayList} of VisualComponentConnection components
     */
    public void setExistingMappings(ArrayList<VisualComponentConnection> mappings) {
        this.existingMappings = mappings;
        this.repaint();
    }

    public ArrayList<VisualComponentConnection> getAddedMappings() {
        return addedMappings;
    }

    public ArrayList<VisualComponentConnection> getDeletedMappings() {
        return deletedMappings;
    }

    public ArrayList<VisualComponentConnection> getMappingsFromHistory(){
        return history.getCanceledAndDeletedConnections();
    }
    /**
     *
     * @return An integer indicating the number of {@link VisualComponentConnection} components in this Container
     */
    public int getExistingMappingsCount() {
        return this.existingMappings.size();
    }

    /**
     * Clears the {@link VisualComponentConnection} {@link ArrayList} of this Container
     */
    public void clearMappings() {
        this.existingMappings.clear();
        this.addedMappings.clear();
        this.deletedMappings.clear();

        this.history.ClearHistory();
    }

    /**
     * Sets the text in the <code>sourceViewLabel</code> and <code>destViewLabel</code>
     * to <code>source</code> and <code>dest</code> respectively
     *
     * @param source The text for sourceViewLabel
     * @param dest The text for destViewLabel
     */
    public void setViewLabels(String source, String dest) {
        sourceViewLabel.setText(source);
        targetViewLabel.setText(dest);
    }

    /**
     * Returns the current line type of all the <code>VisualComponentConnection</code> objects
     * in this Container
     * @return An integer indicating the line type
     */
    public int getLineType() {
        return this.lineType;
    }

    /**
     * Sets the line type of all {@link VisualComponentConnection} components to <code>lineType</code>.
     * <ul>The line type can be one of: <li>CONNECT_LINE_TYPE_SIMPLE
     * <li>CONNECT_LINE_TYPE_RECTANGULAR</ul>
     *
     * @param lineType The line type
     */
    public void setLineType(int lineType) {
        this.lineType = lineType;
        for (int i = 0; i < this.existingMappings.size(); i++) {
            this.existingMappings.get(i).setLineType(lineType);
        }
        for (int i = 0; i < this.deletedMappings.size(); i++) {
            this.deletedMappings.get(i).setLineType(lineType);
        }
        for (int i = 0; i < this.addedMappings.size(); i++) {
            this.addedMappings.get(i).setLineType(lineType);
        }
    }

    /**
     * Returns the current line color of all existing <code>VisualComponentConnection</code> objects
     * in this Container
     * @return An instance of <code>Color</code> class indicating the current color of existing Mappings
     */
    public Color getLineColor() {
        return this.lineColor;
    }

    /**
     * Sets the line color of all existing Mappings to <code>lineColor</code>
     *
     * @param lineColor An instance of <code>Color</code>
     */
    public void setLineColor(Color lineColor) {
        this.lineColor = lineColor;
        for (int i = 0; i < this.existingMappings.size(); i++) {
            this.existingMappings.get(i).setLineColor(lineColor);
        }
    }

    public void setAddedLineColor(Color lineColor) {
        this.addedLineColor = lineColor;
        for (int i = 0; i < this.addedMappings.size(); i++) {
            this.addedMappings.get(i).setLineColor(lineColor);
        }
    }

    public void setDeletedLineColor(Color lineColor) {
        this.deletedLineColor = lineColor;
        for (int i = 0; i < this.deletedMappings.size(); i++) {
            this.deletedMappings.get(i).setLineColor(lineColor);
        }
    }

    public Color getAddedLineColor() {
        return addedLineColor;
    }

    public Color getDeletedLineColor() {
        return deletedLineColor;
    }

    /**
     *
     * @return The <code>source JTree</code>, at the left side of the Container
     */
    public JTree getSourceTree() {
        return this.sourceTree;
    }

    public Point getSourceTreeLocation() {
        return sourceTreeLocation;
    }

    public void setSourceTreeLocation(Point newSourceTreeLocation) {
        this.sourceTreeLocation = newSourceTreeLocation;
    }

    /**
     * Sets the {@link DefaultTreeModel TreeModel} of {@code sourceTree} to {@code treeModel}
     * @param treeModel An instance of {@link DefaultTreeModel}
     */
    public void setSourceTreeModel(DefaultTreeModel treeModel) {
        this.sourceTree.setModel(treeModel);
        TreeUtils.ExpandJTree(this.sourceTree);
    }

    public JScrollPane getSourceTreeScrollPane() {
        return sourceTreeScrollPane;
    }

    public Point getSourceScrollPaneLocation() {
        return sourceScrollPaneLocation;
    }

    public void setSourceScrollPaneLocation(Point newLocation) {
        this.sourceScrollPaneLocation = newLocation;
    }

    /**
     *
     * @return The <code>destination JTree</code>, at the right side of the Container
     */
    public JTree getDestTree() {
        return this.destTree;
    }

    public Point getDestTreeLocation() {
        return destTreeLocation;
    }

    public void setDestTreeLocation(Point newDestTreeLocation) {
        this.destTreeLocation = newDestTreeLocation;
    }

    /**
     * Sets the {@link DefaultTreeModel TreeModel} of {@code destTree} to {@code treeModel}
     * @param treeModel An instance of {@link DefaultTreeModel}
     */
    public void setDestTreeModel(DefaultTreeModel treeModel) {
        this.destTree.setModel(treeModel);
        TreeUtils.ExpandJTree(destTree);
    }

    public Point getDestScrollPaneLocation() {
        return destScrollPaneLocation;
    }

    public void setDestScrollPaneLocation(Point newLocation) {
        this.destScrollPaneLocation = newLocation;
    }

    public JScrollPane getDestTreeScrollPane() {
        return destTreeScrollPane;
    }

    public JButton getUndoButton() {
        return undoButton;
    }

    public JButton getRedoButton() {
        return redoButton;
    }

    /**
     * A temporary method for testing if it's possible to get data from
     * Mappings added by user.
     *
     * @return A JTextArea showing all the Mappings with
     * their source and destination node names.
     */
    public JTextArea getData() {
        JTextArea textData = new JTextArea();
        textData.setEditable(false);

        if (!this.existingMappings.isEmpty() || !this.deletedMappings.isEmpty() || !this.addedMappings.isEmpty()) {
            textData.append("Remaining Mappings:\n");
            for (int i = 0; i < this.existingMappings.size(); i++) {
                textData.append("Mapping: " + (i + 1) + " from: " + this.existingMappings.get(i).getSourcePath().toString() + " to: " + this.existingMappings.get(i).getDestPath().toString() + "\n");
            }
            textData.append("\nAdding Mappings:\n");
            for (int i = 0; i < this.addedMappings.size(); i++) {
                textData.append("Mapping: " + (i + 1) + " from: " + this.addedMappings.get(i).getSourcePath().toString() + " to: " + this.addedMappings.get(i).getDestPath().toString() + "\n");
            }
            textData.append("\nDeleting Mappings:\n");
            for (int i = 0; i < this.deletedMappings.size(); i++) {
                textData.append("Mapping: " + (i + 1) + " from: " + this.deletedMappings.get(i).getSourcePath().toString() + " to: " + this.deletedMappings.get(i).getDestPath().toString() + "\n");
            }
            return textData;
        } else {
            return null;
        }

    }

    @Override
    public void paint(Graphics g) {
        super.paint(g);
        for (int i = 0; i < this.existingMappings.size(); i++) {
            if (this.existingMappings.get(i) != null) {
                this.existingMappings.get(i).paint(g);
            }
        }
        for (int i = 0; i < this.deletedMappings.size(); i++) {
            if (this.deletedMappings.get(i) != null) {
                this.deletedMappings.get(i).paint(g);
            }
        }
        for (int i = 0; i < this.addedMappings.size(); i++) {
            if (this.addedMappings.get(i) != null) {
                this.addedMappings.get(i).paint(g);
            }
        }
    }
    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JButton collapseDestTreeButton;
    private javax.swing.JButton collapseSourceTreeButton;
    private javax.swing.JButton completeButton;
    private org.ow2.choreos.gui.interfaceMappingsVisualizer.components.MappingsLineColorChooser mappingsColorChooser;
    private javax.swing.JTree destTree;
    private javax.swing.JScrollPane destTreeScrollPane;
    private javax.swing.JButton expandDestTreeButton;
    private javax.swing.JButton expandSourceTreeButton;
    private javax.swing.JButton redoButton;
    private javax.swing.JTree sourceTree;
    private javax.swing.JScrollPane sourceTreeScrollPane;
    private javax.swing.JLabel sourceViewLabel;
    private javax.swing.JLabel targetViewLabel;
    private javax.swing.JButton undoButton;
    // End of variables declaration//GEN-END:variables
}
