/**
 * @author Kostas Lekkas -- lekkas@vtrip.net
 */
package org.ow2.choreos.executable_service_discovery.DummyActiveRegistry;

import java.io.File;
import java.io.FileFilter;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.net.Socket;
import java.security.MessageDigest;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.concurrent.ConcurrentLinkedQueue;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;


public class DirectoryScanThread implements Runnable {
    
    private String path;
    private long millis;
    
    private LinkedList<RegistryEntry> currentServices;
    private LinkedList<RegistryEntry> registeredServices;
    private ConcurrentLinkedQueue<Socket> newConnectionQ;
    private LinkedList<Socket> connectionList;
    private Logger logger;
    
    private StringBuffer fileBuf;
    
    public DirectoryScanThread(String path, long millis) {
    	logger = LoggerFactory.getLogger(ConnectionAcceptThread.class);
        logger.info("new thread");
        this.path = path;
        this.millis = millis;
        
        currentServices = Registry.getInstance().getCurrentServices();
        registeredServices = Registry.getInstance().getRegisteredServices();
        connectionList = Registry.getInstance().getConnectionList();
        newConnectionQ = Registry.getInstance().getNewConnectionQ();
    }
    
    @Override
    public void run() {
        Iterator<RegistryEntry> it;
        File dir = new File(path);
        
        /*
         * Add new client connections to connectionList
         * and send them ALL currently registered services
         * on connection.
         */
        while(!newConnectionQ.isEmpty()) {
            Socket sock = newConnectionQ.remove();
            connectionList.add(sock);
            it = registeredServices.iterator();
            while(it.hasNext()) {
                RegistryEntry entry = (RegistryEntry)it.next();
                sendNewMsgToClient(entry, sock);
            }
        }
        
        if(!dir.isDirectory()) {
            logger.error(path+" is not a directory.");
            return;
        }
        File[] list = dir.listFiles(new FileFilter() {
            @Override
            public boolean accept(File pathname) {
                if(pathname.getName().endsWith(".wsdl"))
                    return true;
                return false;
            }
        });

        /*
         * Scan directory and save all current entries.
         */
        for(int i = 0; i < list.length; i++) {
            try {
                String fname = list[i].getCanonicalPath();
                RegistryEntry entry = new RegistryEntry(fname, 
                        getMD5Checksum(fname));
                /*
                 * fileBuf already has the contents of the wsdl description
                 * file. (No need to parse it again)
                 */
                entry.setWsdl(fileBuf.toString());
                currentServices.add(entry);
            } catch (Exception ex) {
                logger.error("Error: "+ex.toString());
            }
        }

        /*
         * check for updated and deleted services
         */
        it = registeredServices.iterator();
        while(it.hasNext()) {
            boolean foundEl = false;
            RegistryEntry entry = (RegistryEntry)it.next();

            Iterator it2 = currentServices.iterator();
            while(it2.hasNext() && !foundEl) {
                RegistryEntry entry2 = (RegistryEntry)it2.next();
                if( entry.getFilename().equals(entry2.getFilename()) ) { 
                    if(entry.getMD5Hash().equals(entry2.getMD5Hash())) { //  SERVICE UNCHANGED
                        it2.remove();
                    }
                    else {  // SERVICE UPDATED
                        entry.setMD5Hash(entry2.getMD5Hash());
                        entry.setWsdl(getContents(entry2.getFilename()));
                        sendUpdateMsgToAll(entry);
                        it2.remove();
                    }
                    foundEl = true;
                }
            }
            if(!foundEl) { // SERVICE DELETED
                sendDeleteMsgToAll(entry);
                it.remove();
            }
        }

        /*
         * register new services
         */
        it = currentServices.iterator();
        while(it.hasNext()) {
            RegistryEntry entry = (RegistryEntry)it.next();
            registeredServices.add(entry);
            sendNewMsgToAll(entry);
            it.remove();
        }
    }
    
    private String getContents(String fname) {
        File f = new File(fname);
        StringBuffer buf = new StringBuffer();
        try {
            InputStreamReader in = new InputStreamReader(new FileInputStream(f), "UTF-8");

            int ch;
            ch = in.read();
            while(ch != -1) {
                buf.append((char)ch);
                ch = in.read();
            }
            in.close();
            logger.info("Parsed contents of file: "+fname);
            return buf.toString();
            
        } catch (FileNotFoundException ex) {
            ex.printStackTrace();
        } catch(IOException ex) {
            ex.printStackTrace();
        }
        return "";
    }
    
    private byte[] createChecksum(String filename) throws Exception {
       InputStream fis =  new FileInputStream(filename);
       fileBuf = new StringBuffer();
       
       byte[] buffer = new byte[1024];
       MessageDigest complete = MessageDigest.getInstance("MD5");
       int numRead;

       do {
           numRead = fis.read(buffer);
           String tmpstr = new String(buffer);
           fileBuf.append(tmpstr.trim());
           
           if (numRead > 0) {
               complete.update(buffer, 0, numRead);
           }
       } while (numRead != -1);

       fis.close();
       return complete.digest();
   }

   private String getMD5Checksum(String filename) throws Exception {
       byte[] b = createChecksum(filename);
       String result = "";

       for (int i=0; i < b.length; i++) {
           result += Integer.toString( ( b[i] & 0xff ) + 0x100, 16).substring( 1 );
       }
       return result;
   }
   
   private void sendUpdateMsgToAll(RegistryEntry entry) {
       logger.info("updated entry: "+entry.getFilename() + " "+entry.getMD5Hash());
       String msg = RegistryMessage.getMsg("UPDATEDENTRY", entry.getFilename(), entry.getMD5Hash(), entry.getWsdl());
       sendMsgToClients(msg);
   }
   
   private void sendDeleteMsgToAll(RegistryEntry entry) {
       logger.info("deleted entry: "+entry.getFilename() + " "+entry.getMD5Hash());
       String msg = RegistryMessage.getMsg("DELETEDENTRY", entry.getFilename(), entry.getMD5Hash(), entry.getWsdl());
       sendMsgToClients(msg);
   }
   
   private void sendNewMsgToAll(RegistryEntry entry) {
       logger.info("new entry: "+entry.getFilename() + " "+entry.getMD5Hash());
       String msg = RegistryMessage.getMsg("NEWENTRY", entry.getFilename(), entry.getMD5Hash(), entry.getWsdl());
       sendMsgToClients(msg);
   }
   
   private void sendNewMsgToClient(RegistryEntry entry, Socket sock) {
       String msg = RegistryMessage.getMsg("NEWENTRY", entry.getFilename(), entry.getMD5Hash(), entry.getWsdl());
       try {
            sock.getOutputStream().write(msg.getBytes());
            logger.info("Sent msg");
        } catch (IOException ex) {
            /**/
        }
   }
   
   private void sendMsgToClients(String msg) {
       Iterator it = connectionList.iterator();
       while(it.hasNext()) {
           Socket sock = (Socket)it.next();
            try {
                sock.getOutputStream().write(msg.getBytes());
                logger.info("Sent msg");
            } catch (IOException ex) {
                logger.error("Connection error. removing client from connections list: "+ex.toString());
                it.remove();
            }
       }
   }
}
