/**
 * @author Kostas Lekkas -- lekkas@vtrip.net
 */
package org.ow2.choreos.executable_service_discovery.plugin_manager;

import org.ow2.choreos.executable_service_discovery.AoSBM.PluginManagerCallback;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.lang.reflect.Constructor;
import java.lang.reflect.InvocationTargetException;
import java.util.HashMap;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.Map.Entry;
import java.util.Properties;
import java.util.concurrent.LinkedBlockingQueue;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

public class PluginManager implements ManagePlugins, ManageServices, PassiveDiscovery {
    private PluginManagerCallback callback;
    private static final PluginManager INSTANCE = new PluginManager("./conf/plugin.properties");
    private static String confDir;
    private Logger logger;
    private LinkedBlockingQueue<PluginMsg> pluginMsgRXQueue;
    private PluginMsgReader readerThread;
    
    /*
     * The 'plugins' hashmap maps each plugin type (which is uniquely 
     * identified by its full class name) to an 'internal' hashmap. This 
     * 'internal' hashmap contains all plugin instances of the specific 
     * plugin type. Each plugin instance is uniquely identified by the 
     * URI of its associated registry.
     */
    private HashMap<String, HashMap<String, AbstractPlugin>> plugins;
    
    private PluginManager(String conf) {
        logger = LoggerFactory.getLogger(PluginManager.class);
        plugins = new HashMap<String, HashMap<String,AbstractPlugin>>();
        pluginMsgRXQueue = new LinkedBlockingQueue<PluginMsg>();
        readerThread = null;
        confDir = conf; 
    }

    /**
     * Returns instance of singleton PluginManager
     * @return 
     */
    public static PluginManager getInstance() {
        return INSTANCE;
    }

    @Override
    public void setRetrieveServiceCallback(PluginManagerCallback callback) {
        this.callback = callback;
    }

    @Override
    public void retrieveService(ServiceFilter service_filter, TargetEnvironment env) {
        /* Call retrieveServices() method on all plugins that are
         * a) supporting active Discovery 
         * b) listed in TargetEnvironment
         */
    }

    @Override
    public void register(CommonServiceDescription service_description, String registryURI) {
    	pluginMsgRXQueue.offer(new PluginMsg(PluginMsg.MSG_REGISTER, service_description, registryURI));
        //callback.register(service_description);
    }
    
    @Override
    public void unregisterByServiceURI(String serviceURI, String registryURI) {
    	pluginMsgRXQueue.offer(new PluginMsg(PluginMsg.MSG_UNREGISTER, serviceURI, registryURI));
        //callback.unregisterByServiceURI(serviceURI);
    }

    
    @Override
    public void addPluginsFromConf() {
        Properties props = new Properties();
        try {
            props.load(new FileInputStream(confDir));
            logger.info("Properties file is loaded.");

            for(Entry<Object, Object> e : props.entrySet())
                if( ((String)e.getValue()).toLowerCase().equals("enabled"))
                    addPluginType( (String)e.getKey() );

        } catch (FileNotFoundException e) {
            logger.error(e.getMessage());
        } catch (IOException e) {
            logger.error(e.getMessage());
        }
    }

    @Override
    public boolean addPluginType(String pluginClassName) {
        try {
            Class<?> Plugin = Class.forName(pluginClassName);
            
            if(!plugins.containsKey(Plugin.getCanonicalName())) {
                plugins.put(Plugin.getCanonicalName(), null);
                logger.info("Added new class type:" +Plugin.getCanonicalName());
                return true;
            }
            else {
                logger.info("plugin already loaded");
                return false;
            }
        } catch (ClassNotFoundException ex) {
            logger.info("Plugin "+pluginClassName+" not found");
            return false;
        }
    }

    @Override
    public LinkedList<String> getAvailablePluginTypes() {
        LinkedList<String> list = new LinkedList<String>();
        Properties props = new Properties();
        try {
            props.load(new FileInputStream(confDir));
            Iterator<Object> it = props.keySet().iterator();
            while(it.hasNext()) {
                String key = (String)it.next();
                String val = (String)props.get(key);
                if(val.toLowerCase().equals("enabled")) 
                    list.add(key);
            }
        } catch (IOException e) {
            logger.error(e.getMessage());
        }
        return list;
    }

    @Override
    public LinkedList<String> getLoadedPluginTypes() {
        LinkedList<String> list = new LinkedList<String>();
        Iterator<String> it = plugins.keySet().iterator();
        while(it.hasNext()) {
            list.add(it.next());
        }
        return list;
    }

    @Override
    public void removeAllPluginTypes() {
        /* unimplemented */
    }

    @Override
    public void removePluginType(String pluginClassName) {
        /* unimplemented */
    }

    @Override
    public boolean createPluginInstance(String pluginClassName, String registryURI) {
        if(!plugins.containsKey(pluginClassName)) {
            logger.error("PluginType "+pluginClassName+" not loaded.");
            return false;
        }
        else {
            HashMap<String, AbstractPlugin> pluginInstances = plugins.get(pluginClassName);
            if(pluginInstances == null) {
                pluginInstances = new HashMap<String, AbstractPlugin>();
                plugins.put(pluginClassName, pluginInstances);
            }
            Class<?> Plugin;
            try {
                Plugin = Class.forName(pluginClassName);
            } catch (ClassNotFoundException e) {
                logger.error(e.getMessage());
                return false;
            }
            Class<?> partypes[] = new Class[1];
            partypes[0] = String.class;
            try {
                Constructor<?> ct = Plugin.getConstructor(partypes);
                Object arglist[] = new Object[1];
                arglist[0] = registryURI;
                Object pluginInstance = ct.newInstance(arglist);
                pluginInstances.put(registryURI, (AbstractPlugin)pluginInstance);
                
                ((AbstractPlugin)pluginInstance).start();
                logger.info("Plugin instance "+registryURI+"started");
                
                if(pluginInstances.size() == 1) 
                    startReaderThread();
                
            } catch (SecurityException e) {
                logger.error(e.toString());
                return false;
            } catch (NoSuchMethodException e) {
                logger.error(e.toString());
                return false;
            } catch (IllegalArgumentException e) {
                logger.error(e.toString());
                return false;
            } catch (InstantiationException e) {
                logger.error(e.toString());
                return false;
            } catch (IllegalAccessException e) {
                logger.error(e.toString());
                return false;
            } catch (InvocationTargetException e) {
                logger.error(e.toString());
                return false;
            }
            return true;
        }
    }
    
    @Override
    public void destroyPluginInstance(String pluginClassName, String registryURI){
        if(!plugins.containsKey(pluginClassName)) {
            logger.error("PluginType "+pluginClassName+" not loaded.");
        }
        else {
            HashMap<String, AbstractPlugin> pluginInstances = plugins.get(pluginClassName);
            if(pluginInstances == null) {
                logger.error("Could not find plugin with uri: "+registryURI);
            }
            else {
                AbstractPlugin plugin = pluginInstances.remove(registryURI);
                if(plugin != null) {
                    plugin.stop();
                    plugin = null;
                    if(pluginInstances.size() == 0) 
                        stopReaderThread();
                        
                }
                else {
                    logger.error("Could not find plugin with uri: "+registryURI);
                }
            }
        }
    }

    @Override
    public LinkedList<String> getPluginInstancesOfType(String pluginClassName) {
        LinkedList<String> list = new LinkedList<String>();

        if(!plugins.containsKey(pluginClassName)) {
            logger.error("PluginType "+pluginClassName+" not loaded.");	
            return list;
        }
        else {
            HashMap<String, AbstractPlugin> pluginInstances = plugins.get(pluginClassName);
            if(pluginInstances == null)
                return list;	// returns empty list
            else {
                Iterator<String> it = pluginInstances.keySet().iterator();
                while(it.hasNext()) 
                    list.add(it.next());
            }
        }
        return list;
    }
    
    
    private void startReaderThread() {
        readerThread = new PluginMsgReader();
        readerThread.start();
    }
    
    private void stopReaderThread() {
        readerThread.interrupt();
        readerThread = null;
    }
    
    private class PluginMsgReader extends Thread {
        @Override
        public void run() {
            logger.info("Started reader thread");
            while(!Thread.currentThread().isInterrupted()) {
                try {
                    PluginMsg msg = pluginMsgRXQueue.take();
                    if(msg == null)
                        logger.error("MSG == NULL??");
                    
                    switch(msg.getMsgType()) {
                        case PluginMsg.MSG_REGISTER: {
                            callback.register(msg.getServiceDescription(), msg.getRegistryURI());
                            break;
                        }
                        case PluginMsg.MSG_UNREGISTER: {
                            callback.unregisterByServiceURI(msg.getServiceURI(), msg.getRegistryURI());
                            break;
                        }
                        default: {
                            logger.error("Unknown msg type : "+msg.getMsgType()+"???");
                            break;
                        }
                    }
                } catch (InterruptedException ex) {
                    logger.info("Reader Thread interrupted: ");
                    pluginMsgRXQueue.clear();
                }
            }
        }
    }
    
    public static void setConfDir(String conf) {
    	confDir = conf;
    }
    
    public static String getConfDir() {
    	return confDir;
    }
}
