package org.ow2.choreos.statistics;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Map.Entry;

import org.apache.commons.math3.distribution.NormalDistribution;
import org.ow2.chereos.iots.common.BorderPoints;
import org.ow2.chereos.iots.common.Location;
import org.ow2.chereos.iots.common.Point;

// in this class, we consider that x and y have the same variance and therefore when we get the range
//in the getLookupArary, we use it to get both x and y values
// 
public class StatUtil {
	/**
	 * The method takes returns an arraylist containing the ranges under the PDF
	 * that satisfy the required area portions
	 * 
	 * 
	 * @param areaRatio
	 *            : the ratio of the p-quantile
	 * @param distance
	 *            : the total distance to be able to specify what the variance
	 *            value should be
	 * @return a hashmap containing the distances from the mean for each subarea
	 */
	public static List<Double> getLookupArray(double areaRatio, double distance) {
		double newRange, oldRange = 0;
		List<Double> lookupArray = new ArrayList<Double>();
		NormalDistribution nDist = new NormalDistribution(0, distance / 2);
		// to start from the 5th P-quantile
		for (double i = (5 + areaRatio / 10); i < 10; i = i + (areaRatio / 10)) {
			newRange = nDist.inverseCumulativeProbability(i / 10);
			lookupArray.add(newRange);

			oldRange = lookupArray.get(lookupArray.size() - 1);
			// System.out.println(i);
		}
		// System.out.println(" size " + lookupArray.size());

		return lookupArray;

	}

	/**
	 * the method is used to construct a grid made of concentric unequal
	 * rectangles from which sensor nodes should be selected
	 * 
	 * @param eventOfInterestPoint
	 *            : the location of the event of interest around which the
	 *            concentric rectangles will be constructed
	 * @param lookupArray
	 *            : the array list containing the computed ranges that satisfy
	 *            the required area ratios under the PDF curve
	 * @return a HashMap containing the id of each square and the border points
	 *         of each rectangle
	 */
	public static Map<Integer, BorderPoints> getGridPoints(
			List<Double> lookupArray, Point eventOfInterestPoint) {
		Map<Integer, BorderPoints> gridPoints = new HashMap<Integer, BorderPoints>();
		int index = 0;
		for (double range : lookupArray) {
			// System.out.println(lookupArray);
			index++;
			// System.out.println("range for i" + index + "is " + range);
			double xMin = eventOfInterestPoint.getX() - range;

			double xMax = eventOfInterestPoint.getX() + range;
			double yMin = eventOfInterestPoint.getY() - range;
			double yMax = eventOfInterestPoint.getY() + range;
			gridPoints.put(index, new BorderPoints(new Point(xMin, yMin),
					new Point(xMax, yMin), new Point(xMin, yMax), new Point(
							xMax, yMax)));

		}

		return gridPoints;

	}

	/**
	 * The method create the grid from which nodes should be uniformly selected
	 * by dividing the area of interest into a set of equal square starting from
	 * the bottom left coordinates of the area of interst
	 * 
	 * @param areaOfInterest
	 *            : the area within which measurements should take place
	 * @param ratio
	 *            : the ratio by which the side of the area should be divided to
	 *            create the grid squares
	 * @return: the grid points starting from the bottom left of the area of
	 *          interest with index 0
	 */
	public static Map<Integer, BorderPoints> getUniformGridPoints(
			BorderPoints areaOfInterest, double ratio) {
		Map<Integer, BorderPoints> gridPoints = new HashMap<Integer, BorderPoints>();
		double width = areaOfInterest.getWidth();
		double height = areaOfInterest.getHeight();
		int counter = 0;
		for (int i = 0; i <= Math.ceil(100 / ratio) - 1; i++) {
			for (int j = 0; j <= Math.ceil(100 / ratio) - 1; j++) {

				gridPoints.put(counter, new BorderPoints(new Point(
						areaOfInterest.getBottomLeftPoint().getX() + i * width
								* ratio / 100, areaOfInterest
								.getBottomLeftPoint().getY()
								+ j
								* height
								* ratio / 100), new Point(areaOfInterest
						.getBottomLeftPoint().getX()
						+ (i + 1)
						* width
						* ratio
						/ 100, areaOfInterest.getBottomLeftPoint().getY() + j
						* height * ratio / 100), new Point(areaOfInterest
						.getBottomLeftPoint().getX() + i * width * ratio / 100,
						areaOfInterest.getBottomLeftPoint().getY() + (j + 1)
								* height * ratio / 100), new Point(
						areaOfInterest.getBottomLeftPoint().getX() + (i + 1)
								* width * ratio / 100, areaOfInterest
								.getBottomLeftPoint().getY()
								+ (j + 1)
								* height
								* ratio / 100)));
				counter++;

			}
		}
		return gridPoints;
	}

	/**
	 * The method takes a hashmap containing the device ids and paths of phones
	 * that measure the requested concept in the region of interest and we
	 * estimate their current location
	 * 
	 * @param fullMap
	 */
	public static Map<String, Location> interpolateAllServices(
			Map<String, List<Location>> fullMap, long t) {

		Map<String, Location> interpolatedMap = new HashMap<String, Location>();
		for (Entry<String, List<Location>> entry : fullMap.entrySet()) {
			List<Location> locations = entry.getValue();

			int index = lowerBinarySearch(locations, 0, locations.size() - 1, t);
			Location selectedLocation = locations.get(index);
			Location locationNow = null;
			if (index < 0)
				continue;
			else if (selectedLocation.getTime() == t) {
				locationNow = selectedLocation;
			} else if (index == locations.size() - 1)
				continue;
			else {
				locationNow = interpolate(selectedLocation,
						locations.get(index + 1), t);
				interpolatedMap.put(entry.getKey(), locationNow);
			}
		}
		return interpolatedMap;
	}

	private static Location interpolate(Location minTimeLocation,
			Location maxTimeLocation, long time) {

		double alpha = ((double) (time - minTimeLocation.getTime()))
				/ ((maxTimeLocation.getTime() - minTimeLocation.getTime()));

		return new Location(
				minTimeLocation.getXCoordinate()
						+ alpha
						* (maxTimeLocation.getXCoordinate() - minTimeLocation
								.getXCoordinate()),
				minTimeLocation.getYCoordinate()
						+ alpha
						* (maxTimeLocation.getYCoordinate() - minTimeLocation
								.getYCoordinate()), time);
	}

	/**
	 * The method takes the path of registered services and estimates the
	 * location of the registered services at a specific time t
	 * 
	 * @param registeredSL
	 *            : the path of the registered services
	 * @param t
	 *            : the time at which we want to estimate the location
	 * @return Location: the estimated location
	 */
	public static Location interpolateOneService(
			ArrayList<Location> registeredSL, long t) {

		int index = lowerBinarySearch(registeredSL, 0, registeredSL.size() - 1,
				t);
		Location locationNow = null;
		if(index != -1){
		Location selectedLocation = registeredSL.get(index);
	
		if (index < 0) {

		} else if (selectedLocation.getTime() == t) {
			locationNow = selectedLocation;
		}

		else if (index == registeredSL.size() - 1) {

		}

		else {
			locationNow = interpolate(selectedLocation,
					registeredSL.get(index + 1), t);
		}
		}
		return locationNow;
	}

	/**
	 * The method gets the index of the biggest time that is smaller than time
	 * now
	 * 
	 * @param arr
	 *            the array of locations of nodes, each location has a time
	 * @param minIndex
	 * @param maxIndex
	 * @param t
	 *            : the time to which we are comparing times in the array, i.e.
	 *            time now
	 * @return
	 */
	public static int lowerBinarySearch(List<Location> arr, int minIndex,
			int maxIndex, long t) {

		if (arr == null || arr.size() < maxIndex + 1 || minIndex < 0
				|| maxIndex < minIndex) {
			// basic error checks
			return -1;
		} else {
			int tempLength = maxIndex - minIndex + 1;

			if (tempLength == 1) {

				if (arr.get(minIndex).getTime() <= t)
					return minIndex;
				else
					// the value is larger
					return minIndex - 1; // this will also return -1 if the
											// minIndex give was 1, so all is
											// well
			} else if (tempLength == 2) {
				// maxIndex is minIndex + 1, fun times
				if (arr.get(minIndex).getTime() > t) {
					// they are all larger
					return minIndex - 1; // this will also return -1 if the
											// minIndex give was 1, so all is
											// well
				} else if (arr.get(maxIndex).getTime() <= t) {
					// min is smaller, and so is max, so we return max
					return maxIndex;
				} else {
					// min is smaller, but max is larger. we return min
					return minIndex;
				}
			} else {

				// tempLength is larger than 2, time to split
				int splitIndex = minIndex + tempLength / 2; // integer division

				if (arr.get(splitIndex).getTime() == t) {
					return splitIndex;
				} else if (arr.get(splitIndex).getTime() > t) {
					// check in lower array
					return lowerBinarySearch(arr, minIndex, splitIndex - 1, t);
				} else {
					// split point still smaller
					// check in upper array
					return lowerBinarySearch(arr, splitIndex + 1, maxIndex, t);
				}
			}

		}
	}

	/**
	 * The method takes the map containing the address of all services that
	 * measure a concept and all their locations it uses dmax and the required
	 * size of the subset and the location of interest to generate a subset of
	 * services that are normally distributed around the locations of interest,
	 * by dividing the area into concentric square and selecting an equal number
	 * of sensors in each rectangle. important to note that the difference
	 * between the square and the ones inside it as the grid square
	 * 
	 * @param fullMap
	 * @param dMax
	 * @param locOfInterest
	 * @param subsetSize
	 * @return
	 */
	public static List<String> getFinalSubSet(
			Map<String, List<Location>> fullMap, double dMax,
			Point locOfInterest, int subsetSize, double ratio, long t) {
		Map<String, Location> oneLocPerDeviceMap = new HashMap<String, Location>();
		List<Double> lookupArray = new ArrayList<Double>();
		Map<Integer, BorderPoints> gridMap = new HashMap<Integer, BorderPoints>();
		Map<Integer, List<String>> serviceSubSetGrid = new HashMap<Integer, List<String>>();
		oneLocPerDeviceMap = StatUtil.interpolateAllServices(fullMap, t);
		lookupArray = getLookupArray(ratio, dMax);
		gridMap = getGridPoints(lookupArray, locOfInterest);

		serviceSubSetGrid = StatUtil.identifySquares(oneLocPerDeviceMap,
				gridMap);

		List<String> finalDeviceSubList = new ArrayList<String>();
		List<String> tempArray = new ArrayList<String>();
		List<String> newArray = new ArrayList<String>();
		for (Entry<Integer, List<String>> entry : serviceSubSetGrid.entrySet()) {

			newArray = entry.getValue();
			boolean result = newArray.removeAll(tempArray);

			tempArray = newArray;
			serviceSubSetGrid.put(entry.getKey(), newArray);
			int counter = 0;
			while (counter < subsetSize / gridMap.size()) {
				
				int index = (int) (Math.ceil(Math.random() * newArray.size())) - 1;
				if (finalDeviceSubList.contains(newArray.get(index)) == false) {
					finalDeviceSubList.add(newArray.get(index));
					counter++;
				}
				if (counter == oneLocPerDeviceMap.size())
					break;
				
			}
		}
		return finalDeviceSubList;
	}

	/**
	 * The method takes the map containing the address of all services that
	 * measure a concept and all their locations it uses the required size of
	 * the subset and the area of interest to generate a subset of services that
	 * are uniformly distributed within the area of interest, by dividing the
	 * area into equal square and selecting an equal number of sensors in each
	 * square. The method is also called if the event of interest is a point has
	 * does not spread and sensors to select should be within an small area of
	 * +/- sensing range and the number of sensors should be 6.
	 * 
	 * @param fullMap
	 * @param areaOfInterest
	 * @param subsetSize
	 * @return
	 */
	public static List<String> getUniformFinalSubSet(
			Map<String, List<Location>> fullMap, BorderPoints areaOfInterest,
			int subsetSize, double ratio, long t) {
		Map<String, Location> oneLocPerDeviceMap = new HashMap<String, Location>();
		Map<Integer, BorderPoints> gridMap = new HashMap<Integer, BorderPoints>();
		Map<Integer, List<String>> serviceSubSetGrid = new HashMap<Integer, List<String>>();
		oneLocPerDeviceMap = StatUtil.interpolateAllServices(fullMap, t);

		gridMap = getUniformGridPoints(areaOfInterest, ratio);

		serviceSubSetGrid = StatUtil.identifySquares(oneLocPerDeviceMap,
				gridMap);
		List<String> finalDeviceSubList = new ArrayList<String>();
		List<String> newArray = new ArrayList<String>();
		for (Entry<Integer, List<String>> entry : serviceSubSetGrid.entrySet()) {

			newArray = entry.getValue();
			serviceSubSetGrid.put(entry.getKey(), newArray);
			double counter = 0;
			while (counter < subsetSize / gridMap.size()) {
				int index = (int) (Math.ceil(Math.random() * newArray.size())) - 1;
				if (finalDeviceSubList.contains(newArray.get(index)) == false) {
					finalDeviceSubList.add(newArray.get(index));
					counter++;
				}
				if (counter == oneLocPerDeviceMap.size())
					break;
				
			}
		}
		return finalDeviceSubList;
	}

	/**
	 * the method is used to determine services belong to which parts of the
	 * constructed grid based on their interpolated location
	 * 
	 * @param oneLocPerDeviceMap
	 * @param gridMap
	 * @return
	 */
	public static Map<Integer, List<String>> identifySquares(
			Map<String, Location> oneLocPerDeviceMap,
			Map<Integer, BorderPoints> gridMap) {
		Map<Integer, List<String>> serviceSubSetGrid = new HashMap<Integer, List<String>>();
		for (Entry<String, Location> entry : oneLocPerDeviceMap.entrySet()) {
			Location thisDeviceLoc = entry.getValue();
			gridLoop: for (Entry<Integer, BorderPoints> gridEntry : gridMap
					.entrySet()) {
				BorderPoints rec = gridEntry.getValue();

				List<String> devicesInRec = new ArrayList<String>();
				if (rec.getBottomLeftPoint().getX() <= thisDeviceLoc
						.getXCoordinate()
						&& rec.getBottomLeftPoint().getY() <= thisDeviceLoc
								.getYCoordinate()
						&& rec.getBottomRightPoint().getX() >= thisDeviceLoc
								.getXCoordinate()
						&& rec.getBottomRightPoint().getY() <= thisDeviceLoc
								.getYCoordinate()
						&& rec.getUpperLeftPoint().getX() <= thisDeviceLoc
								.getXCoordinate()
						&& rec.getUpperLeftPoint().getY() >= thisDeviceLoc
								.getYCoordinate()
						&& rec.getUpperRightPoint().getX() >= thisDeviceLoc
								.getXCoordinate()
						&& rec.getUpperRightPoint().getY() >= thisDeviceLoc
								.getYCoordinate()) {
					if (serviceSubSetGrid.isEmpty() == false)

						devicesInRec = serviceSubSetGrid
								.get(gridEntry.getKey());
					if (devicesInRec == null)
						devicesInRec = new ArrayList<String>();

					String thisAdd = entry.getKey();
					devicesInRec.add(thisAdd);
					serviceSubSetGrid.put(gridEntry.getKey(), devicesInRec);

					break gridLoop;

				}

			}

		}
		return serviceSubSetGrid;
	}

	/**
	 * the method checks if two path intersect at the same time at some
	 * locations and returns those location as covered.
	 * 
	 * @param mainPathPoints
	 *            : the Path of the phone to register
	 * @param estimatedPathPoints
	 *            : the Path of a registered service
	 * @return a list of locations of the path of the phone to registered that
	 *         intersect with locations of a registered service path at the same
	 *         time.
	 */
	public static ArrayList<Location> intersects(List<Location> mainPathPoints,
			ArrayList<Location> estimatedPathPoints, double range) {
		ArrayList<Location> covered = new ArrayList<Location>();

		Location l;
		for (Location lMain : mainPathPoints) {

			long t = lMain.getTime();
			l = interpolateOneService(estimatedPathPoints, t);
			// check if the
			// registered service is within sensing range
			// of the location of my path
			if (l != null)
				if (((lMain.getXCoordinate() - range) < l.getXCoordinate())
						&& ((lMain.getXCoordinate() + range) > l
								.getXCoordinate())
						&& ((lMain.getYCoordinate() - range) < l
								.getYCoordinate())
						&& ((lMain.getYCoordinate() + range) > l
								.getYCoordinate())) {
					covered.add(lMain);

				}

		}

		return covered;
	}
}
