package org.ow2.choreos.registryclient.impl;

import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.OutputStreamWriter;
import java.net.MalformedURLException;
import java.net.URL;
import java.net.URLConnection;
import java.util.ArrayList;
import java.util.HashMap;

import org.json.simple.JSONArray;
import org.json.simple.JSONObject;
import org.json.simple.parser.JSONParser;
import org.json.simple.parser.ParseException;
import org.ow2.choreos.registryclient.interfaces.RegistryClient;

import fr.inria.arles.choreos.iots.common.Service;

public class RestRegistryImpl implements RegistryClient {
	String baseURL;

	public RestRegistryImpl(String urlString) {
		baseURL = urlString;
	}

	@Override
	/**
	 * This method is called by the registration manager to find the number of services measuring concepts of interest
	 * the number should be provided for each concept 
	 * @param myConcepts: the concepts of interest
	 * @param locationOfInterest: the location at which concepts of interest should be measured
	 * @param deviceId: the id of the device hosting the initial service. This is important
	 * because it should be excluded when returning the number of services measuring each concept.
	 * @return the HashMap containing the concept of interest and the number of registered services that
	 * measure this concept.
	 */
	public HashMap<String, Double> findNumberOfServicesReq(
			ArrayList<String> myConcepts, String locationOfInterest,
			String deviceId) {
		HashMap<String, Double> thisMap = new HashMap<String, Double>();
		String paramString = "?device_id=" + deviceId + "&global_region="
				+ locationOfInterest;
		for (int i = 0; i < myConcepts.size(); i++) {
			paramString = paramString + "&concept=" + myConcepts.get(i);

		}
		try {
			URL url = new URL(baseURL + "find_number_of_services" + paramString);

			URLConnection conn = null;
			try {

				conn = url.openConnection();
				conn.setDoOutput(true);
			} catch (IOException e) {
				System.out
						.println("RestRegistryImpl.findNumberOfServices: could not open connection");
				e.printStackTrace();
				return null;
			}

			BufferedReader in = null;
			try {
				in = new BufferedReader(new InputStreamReader(
						conn.getInputStream()));
			} catch (IOException e) {
				System.out
						.println("RestRegistryImpl.findNumberOfServices: could not get inputStream");
				e.printStackTrace();
			//	return null;
				try {
					in = new BufferedReader(new InputStreamReader(
							conn.getInputStream()));
				} catch (IOException ex) {
					System.out
							.println("RestRegistryImpl.findNumberOfServices: could not get inputStream");
					e.printStackTrace();
					return null;
				}
			}

			String response = null;
			try {
				while ((response = in.readLine()) != null) {
					// System.out.println(response);
					JSONParser myParser = new JSONParser();
					try {
						JSONObject responseObject = (JSONObject) myParser
								.parse(response);
						thisMap = (HashMap) responseObject.get("value");

					} catch (ParseException e) {
						System.out
								.println("RestRegistryImpl.findNumberOfServices: returned object is incorrect");
						e.printStackTrace();
						return null;
					}

				}
			} catch (IOException e) {
				System.out
						.println("RestRegistryImpl.findNumberOfServices: could not return a correct response");
				e.printStackTrace();
				return null;
			}
			try {
				in.close();
			} catch (IOException e) {
				System.out
						.println("RestRegistryImpl.findNumberOfServices: could not close inputStream");
				e.printStackTrace();
				return null;

			}

		} catch (MalformedURLException e) {
			System.out
					.println("RestRegistryImpl.findNumberOfServices:URL was malformed");
			e.printStackTrace();
			return null;
		}

		return thisMap;
	}

	@Override
	/**
	 * The method is called to perform the actual registration of the service in a device registry.
	 * if the registry is remote, it invokes a web service.
	 * @param ps: the service to register
	 * @return true if the registration was successful and false otherwise.
	 */
	public boolean registerServiceReq(Service ps, long flightPeriod) {

		boolean result = false;
		try {
			URL url = new URL(baseURL + "register_service");
			URLConnection conn = null;

			try {
				conn = url.openConnection();
				conn.setDoOutput(true);
			} catch (IOException e) {
				System.out
						.println("RestRegistryImpl:registerServiceReq: could not open connection");
				e.printStackTrace();
				return false;
			}

			BufferedWriter out = null;
			try {
				out = new BufferedWriter(new OutputStreamWriter(
						conn.getOutputStream()));
			} catch (IOException e) {
				System.out
						.println("RestRegistryImpl.registerServiceReq: could not create outputStream");
				e.printStackTrace();
				return false;
			}
			try {// TODO send ontology namespace? (to be added to concept and
					// sensor)
				out.write("address=" + ps.getServiceAddress() + "&");
				out.write("device_id=" + ps.getDeviceId() + "&");
				out.write("name=" + ps.getServiceName() + "&");
				out.write("type=" + ps.getServiceType() + "&");
				out.write("accuracy=" + ps.getServiceAccuracy() + "&");
				out.write("sensor=" + ps.getTypeOfServiceSensor() + "&");
				out.write("range=" + ps.getServiceRange() + "&");
				out.write("unit=" + ps.getUnit() + "&");
				out.write("global_region="
						+ ps.getGlobalRegion().getLocationName() + "&");
				out.write("concept=" + ps.getConcept() + "&");
				out.write("dataType=" + ps.getMeasuredDataType() + "&");
				out.write("coordinateX=" + ps.getCoordinateX() + "&");
				out.write("coordinateY=" + ps.getCoordinateY() + "&");
				out.write("expiry_date=" + flightPeriod);
			} catch (IOException e) {
				System.out
						.println("RestRegistryImpl.registerServiceReq: could not write outputStream");
				e.printStackTrace();
				return false;
			}
			try {
				out.flush();
			} catch (IOException e) {
				e.printStackTrace();
				return false;
			}
			try {
				out.close();
			} catch (IOException e) {
				e.printStackTrace();
				return false;
			}
			BufferedReader in = null;
			try {
				in = new BufferedReader(new InputStreamReader(
						conn.getInputStream()));
				try {
					String response = "";

					while ((response = in.readLine()) != null) {
						if (response.equals("true"))
							result = true;
						else
							result = false;
					}
				} catch (IOException e) {
					e.printStackTrace();
					return false;
				}
				try {
					in.close();
				} catch (IOException e) {
					e.printStackTrace();
					return false;
				}

			} catch (MalformedURLException e) {
				e.printStackTrace();
				return false;
			}
		} catch (IOException e) {
			e.printStackTrace();
			return false;
		}

		return result;
	}

	@Override
	/**
	 * This method is called to remove the service from the registry if needed. If the registry is remote,
	 * this method invokes a web service.
	 * @param phoneService: the service to remove. 
	 */
	public void removeServiceReq(Service phoneService) {
		try {
			URL url = new URL(baseURL + "remove_service");
			URLConnection conn = null;

			try {
				conn = url.openConnection();
				conn.setDoOutput(true);
			} catch (IOException e) {
				System.out
						.println("RestRegistryImpl.removeServiceReq: could not open connection");
				e.printStackTrace();
				return;
			}

			BufferedWriter out = null;
			try {
				out = new BufferedWriter(new OutputStreamWriter(
						conn.getOutputStream()));
			} catch (IOException e) {
				System.out
						.println("RestRegistryImpl.removeServiceReq: could not create outputStream");
				e.printStackTrace();
				return;
			}
			try {
				out.write("address=" + phoneService.getServiceAddress());

			} catch (IOException e) {
				System.out
						.println("RestRegistryImpl.removeServiceReq: could not write ouputStream");
				e.printStackTrace();
				return;
			}
			try {
				out.flush();
			} catch (IOException e) {
				e.printStackTrace();
				return;
			}
			try {
				out.close();
			} catch (IOException e) {
				e.printStackTrace();
				return;
			}
			BufferedReader in = null;
			try {
				in = new BufferedReader(new InputStreamReader(
						conn.getInputStream()));
			} catch (IOException e) {
				System.out
						.println("RestRegistryImpl.removeServiceReq: could not get inputStream");
				e.printStackTrace();
				return;
			}

			String response;
			try {
				while ((response = in.readLine()) != null) {
					// System.out.println("the remove service response is :" +
					// response);
				}
			} catch (IOException e) {
				System.out
						.println("RestRegistryImpl.removeServiceReq: could not read inputStream");
				e.printStackTrace();
				return;
			}
			try {
				in.close();
			} catch (IOException e) {
				e.printStackTrace();
				return;
			}

		} catch (MalformedURLException e) {
			System.out
					.println("RestRegistryImpl.removeServiceReq: URL was malformed");
			e.printStackTrace();
			return;
		}

	}

	/**
	 * This method is called by an already registered service to ask the
	 * registry manager to keep it registered for some period of time
	 * 
	 * @param Service
	 *            ps: the service itself
	 * @param flightPeriod
	 *            : the duration to keep the service registered
	 */
	@Override
	public boolean keepServiceReq(Service ps, long flightPeriod) {

		boolean result = false;
		try {
			URL url = new URL(baseURL + "keep_service");
			URLConnection conn = null;

			try {
				conn = url.openConnection();
				conn.setDoOutput(true);
			} catch (IOException e) {
				System.out
						.println("RestRegistryImpl:keepServiceReq: could not open connection");
				e.printStackTrace();
				return false;
			}

			BufferedWriter out = null;
			try {
				out = new BufferedWriter(new OutputStreamWriter(
						conn.getOutputStream()));
			} catch (IOException e) {
				System.out
						.println("RestRegistryImpl.keepServiceReq: could not create outputStream");
				e.printStackTrace();
				return false;
			}
			try {// TODO send ontology namespace? (to be added to concept and
					// sensor)
				out.write("address=" + ps.getServiceAddress() + "&");
				out.write("expiry_date=" + flightPeriod);
			} catch (IOException e) {
				System.out
						.println("RestRegistryImpl.keepServiceReq: could not write outputStream");
				e.printStackTrace();
				return false;
			}
			try {
				out.flush();
			} catch (IOException e) {
				e.printStackTrace();
				return false;
			}
			try {
				out.close();
			} catch (IOException e) {
				e.printStackTrace();
				return false;
			}
			BufferedReader in = null;
			try {
				in = new BufferedReader(new InputStreamReader(
						conn.getInputStream()));
				try {
					String response = "";

					while ((response = in.readLine()) != null) {
						if (response.equals("true"))
							result = true;
						else
							result = false;
					}
				} catch (IOException e) {
					e.printStackTrace();
					return false;
				}
				try {
					in.close();
				} catch (IOException e) {
					e.printStackTrace();
					return false;
				}

			} catch (MalformedURLException e) {
				e.printStackTrace();
				return false;
			}
		} catch (IOException e) {
			e.printStackTrace();
			return false;
		}

		return result;

	}

}
